/*
 * NanoBuffers.h
 *
 */

#ifndef NANOBUFFER_H_
#define NANOBUFFER_H_

#include <stddef.h>
#include <math.h>

class NanoWriter {
private:
	uint8_t* data;
	int size; //
	int pos;

	void _putUInt8 (uint8_t v) {
		resize (pos+1);

		data [pos++] = v;
	}

	void _putUInt64 (uint64_t v) {
		if (v <= 240) {
			_putUInt8 ((uint8_t)v);
		} else if (v <= 2287) {
			_putUInt8 ((uint8_t)((v - 240) / 256 + 241));
			_putUInt8 ((uint8_t)((v - 240) % 256));
		} else if (v <= 67823) {
			_putUInt8 ((uint8_t)249);
			_putUInt8 ((uint8_t)((v - 2288) / 256));
			_putUInt8 ((uint8_t)((v - 2288) % 256));
		} else if (v <= 16777215) {
			_putUInt8 ((uint8_t)250);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
		} else if (v <= 4294967295) {
			_putUInt8 ((uint8_t)251);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 24) & 0xFF));
		} else if (v <= 1099511627775) {
			_putUInt8 ((uint8_t)252);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 24) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 32) & 0xFF));
		} else if (v <= 281474976710655) {
			_putUInt8 ((uint8_t)253);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 24) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 32) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 40) & 0xFF));
		} else if (v <= 72057594037927935) {
			_putUInt8 ((uint8_t)254);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 24) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 32) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 40) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 48) & 0xFF));
		} else {
			_putUInt8 ((uint8_t)255);
			_putUInt8 ((uint8_t)(v & 0xFF));
			_putUInt8 ((uint8_t)((v >> 8) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 16) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 24) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 32) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 40) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 48) & 0xFF));
			_putUInt8 ((uint8_t)((v >> 56) & 0xFF));
		}
	}

	void _putInt64 (int64_t v) {
		_putUInt64 ((uint64_t)((v << 1) ^ (v >> 63)));
	}

	void _putFloat (float v) {
		resize (pos + sizeof (float));

		memcpy((data+pos), &v, sizeof(float));
		pos += sizeof(float);
	}

	void _putDouble (double v) {
		resize (pos + sizeof (double));

		memcpy((data+pos), &v, sizeof(double));
		pos += sizeof(double);
	}

	void _putBytes (uint8_t* v, int len) {
		resize (pos+len);

		memcpy (data+pos, v, len);
		pos += len;
	}

	// check_resize (n)
	int resize (int n) {
		if (n > size) {
			size = size*2;
			if (size < n)
				size = n;

			uint8_t* newData = new uint8_t [size];
			memcpy (newData, data, pos);
			delete [] data;

			data = newData;
		}

		return size;
	}

	void fit () {
		if (pos == 0) {
			if (data)
				delete [] data;
			data = NULL;
			size = 0;
			return;
		}

		size = pos;

		uint8_t* newData = new uint8_t [size];
		memcpy (newData, data, pos);
		delete [] data;

		data = newData;
	}

private:
	NanoWriter& operator= (const NanoWriter& obj);
	NanoWriter (const NanoWriter& obj);

public:
	NanoWriter (int n = 0) {
		size = n;
		if (size < 64)
			size = 64;

		data = new uint8_t [size];
		pos = 0;
	}

	virtual ~NanoWriter() {
		delete [] data;
	}

	int getBytesLeft () {
		return size - pos;
	}

	int getLength () {
		return pos;
	}

	uint8_t* getBytes () {
		fit ();
		return data;
	}

	NanoWriter& putInt (int8_t v) {
		_putInt64 (v);
		return *this;
	}

	NanoWriter& putInt (uint8_t v) {
		_putUInt64 (v);
		return *this;
	}

	NanoWriter& putInt (int16_t v) {
		_putInt64 (v);
		return *this;
	}

	NanoWriter& putInt (uint16_t v) {
		_putUInt64 (v);
		return *this;
	}

	NanoWriter& putInt (int32_t v) {
		_putInt64 (v);
		return *this;
	}

	NanoWriter& putInt (uint32_t v) {
		_putUInt64 (v);
		return *this;
	}

	NanoWriter& putInt (int64_t v) {
		_putInt64 (v);
		return *this;
	}

	NanoWriter& putInt (uint64_t v) {
		_putUInt64 (v);
		return *this;
	}

	NanoWriter& putBytes (uint8_t* v, int len) {
		_putUInt64 (len);
		_putBytes (v, len);
		return *this;
	}

	NanoWriter& putString (const char* v) {
		uint64_t len = strlen (v);

		_putUInt64 (len);
		_putBytes ((uint8_t*) v, len);
		return *this;
	}

	NanoWriter& putFloat (float v, int precision = -1) {
		if (precision < 0)
			_putFloat (v);
		else
			_putInt64 ((int32_t)((v*pow(10, precision+1) + 5)/10));
		return *this;
	}

	NanoWriter& putDouble (double v, int precision = -1) {
		if (precision == -1)
			_putDouble (v);
		else
			_putInt64 ((int64_t)((v*pow(10, precision+1) + 5)/10));
		return *this;
	}
};

class NanoReader {
private:
	uint8_t* data;
	int32_t size;
	int32_t pos;

private:
	uint8_t _getUInt8 () {
		if (getBytesLeft () < 1) {
			// 抛出异常
			return 0;
		}

		return data [pos++];
	}

	char _getInt8 () {
		uint8_t v = _getUInt8 ();

		return (char)((char)(v >> 1) ^ -(char)(v & 1));
	}

	uint16_t _getUInt16() {
		uint8_t a0 = _getUInt8 ();
		if (a0 < 241) {
			return a0;
		}

		uint8_t a1 = _getUInt8 ();
		if (a0 >= 241 && a0 <= 248) {
			return (uint16_t)(240 + 256 * (a0 - 241) + a1);
		}

		uint8_t a2 = _getUInt8 ();
		if (a0 == 249) {
			return (uint16_t)(2288 + 256 * a1 + a2);
		}

		// throw new IndexOutOfRangeException("getUInt16() failure: " + a0);
		return 0;
	}

	int16_t _getInt16() {
		uint16_t v = _getUInt16 ();

		return (int16_t)((int16_t)(v >> 1) ^ -(int16_t)(v & 1));
	}

	uint32_t _getUInt32 () {
		uint8_t a0 = _getUInt8 ();
		if (a0 < 241) {
			return a0;
		}

		uint8_t a1 = _getUInt8 ();
		if (a0 >= 241 && a0 <= 248) {
			return (uint32_t)(240 + 256 * (a0 - 241) + a1);
		}

		uint8_t a2 = _getUInt8 ();
		if (a0 == 249) {
			return (uint32_t)(2288 + 256 * a1 + a2);
		}

		uint8_t a3 = _getUInt8 ();
		if (a0 == 250) {
			return a1 + (((uint32_t)a2) << 8) + (((uint32_t)a3) << 16);
		}

		uint8_t a4 = _getUInt8 ();
		if (a0 >= 251) {
			return a1 + (((uint32_t)a2) << 8) + (((uint32_t)a3) << 16) + (((uint32_t)a4) << 24);
		}

		//throw new IndexOutOfRangeException("getUInt32() failure: " + a0);
		return 0;
	}

	int32_t _getInt32() {
		uint32_t v = _getUInt32 ();

		return (int32_t)((int32_t)(v >> 1) ^ -(int32_t)(v & 1));
	}

	uint64_t _getUInt64 () {
		uint8_t a0 = _getUInt8 ();
		if (a0 < 241) {
			return a0;
		}

		uint8_t a1 = _getUInt8 ();
		if (a0 >= 241 && a0 <= 248) {
			return (uint32_t)(240 + 256 * (a0 - 241) + a1);
		}

		uint8_t a2 = _getUInt8 ();
		if (a0 == 249) {
			return (uint32_t)(2288 + 256 * a1 + a2);
		}

		uint8_t a3 = _getUInt8 ();
		if (a0 == 250) {
			return a1 + (((uint32_t)a2) << 8) + (((uint32_t)a3) << 16);
		}

		uint8_t a4 = _getUInt8 ();
		if (a0 == 251) {
			return a1 + (((uint32_t)a2) << 8) + (((uint32_t)a3) << 16) + (((uint32_t)a4) << 24);
		}

		uint8_t a5 = _getUInt8 ();
		if (a0 == 252) {
			return a1 + (((uint64_t)a2) << 8) + (((uint64_t)a3) << 16) + (((uint64_t)a4) << 24) + (((uint64_t)a5) << 32);
		}

		uint8_t a6 = _getUInt8 ();
		if (a0 == 253) {
			return a1 + (((uint64_t)a2) << 8) + (((uint64_t)a3) << 16) + (((uint64_t)a4) << 24) + (((uint64_t)a5) << 32)
					+ (((uint64_t)a6) << 40);
		}

		uint8_t a7 = _getUInt8 ();
		if (a0 == 254) {
			return a1 + (((uint64_t)a2) << 8) + (((uint64_t)a3) << 16) + (((uint64_t)a4) << 24) + (((uint64_t)a5) << 32)
					+ (((uint64_t)a6) << 40) + (((uint64_t)a7) << 48);
		}

		uint8_t a8 = _getUInt8 ();
		if (a0 == 255) {
			return a1 + (((uint64_t)a2) << 8) + (((uint64_t)a3) << 16) + (((uint64_t)a4) << 24) + (((uint64_t)a5) << 32)
					+ (((uint64_t)a6) << 40) + (((uint64_t)a7) << 48)  + (((uint64_t)a8) << 56);
		}

		// throw new IndexOutOfRangeException("getUInt64 () failure: " + a0);
		return 0;
	}

	int64_t _getInt64() {
		uint64_t v = _getUInt64 ();

		return (int64_t) ((int64_t)(v >> 1) ^ -(int64_t)(v & 1));
	}

	float _getFloat () {
		if (getBytesLeft () < sizeof (float)) {
			// 抛出异常
			return 0;
		}

		float v = 0.0f;
		memcpy(&v, (data+pos), sizeof(float));
		pos += sizeof(float);
		return v;
	}

	double _getDouble () {
		if (getBytesLeft () < sizeof (double)) {
			// 抛出异常
			return 0;
		}

		double v = 0;
		memcpy(&v, (data+pos), sizeof(double));
		pos += sizeof(double);
		return v;
	}

	//
	int32_t _getBytes (uint8_t* buf, uint32_t len) {
		if (len > getBytesLeft ()) {
			// 抛出异常
			return getBytesLeft ();
		}

		memcpy (buf, data+pos, len);
		pos += len;

		return len;
	}

private:
	NanoReader (const NanoReader&);
	NanoReader& operator=(const NanoReader&);

public:
	NanoReader (uint8_t* buf, int len) {
		pos = 0;
		size = len;
		data = new uint8_t[size];

		memcpy (data, buf, len);
	}

	virtual ~NanoReader () {
		if (data)
			delete [] data;
	}

	int32_t getBytesLeft () {
		return size - pos;
	}

	int32_t getLength () {
		return pos;
	}

	NanoReader& getInt (int8_t& v) {
		v = _getInt64 ();

		return *this;
	}

	NanoReader& getInt (uint8_t& v) {
		v = _getUInt64 ();

		return *this;
	}

	NanoReader& getInt (int16_t& v) {
		v = _getInt64 ();

		return *this;
	}

	NanoReader& getInt (uint16_t& v) {
		v = _getUInt64 ();

		return *this;
	}

	NanoReader& getInt (int32_t& v) {
		v = _getInt64 ();

		return *this;
	}

	NanoReader& getInt (uint32_t& v) {
		v = _getUInt64 ();

		return *this;
	}

	NanoReader& getInt (int64_t& v) {
		v =  _getInt64 ();

		return *this;
	}

	NanoReader& getInt (uint64_t& v) {
		v = _getUInt64 ();

		return *this;
	}

	NanoReader& getFloat (float& v, int32_t precision = -1) {
		if (precision < 0)
			v =  _getFloat ();
		else
			v =  ((float) _getInt64 () / pow(10, precision));

		return *this;
	}

	NanoReader& getDouble (double& v, int32_t precision = -1) {
		if (precision < 0)
			v =  _getDouble ();
		else
			v =  ((double) _getInt64 () / pow(10, precision));

		return *this;
	}

	NanoReader& getBytes (uint8_t* buf, int32_t& size) {
		uint64_t len = _getUInt64 ();

		if (len <= size)
			size = _getBytes ((uint8_t*)buf, len);
		else {
			// 错误
		}

		return *this;
	}

	NanoReader& getString (char* buf, int32_t& size) {
		uint64_t len = _getUInt64 ();

		if (len+1 <= size) {
			size = _getBytes ((uint8_t*)buf, len);
			buf [len] = '\0';
		} else {
			// 错误
		}

		return *this;
	}
};

#endif /* NANOBUFFER_H_ */
