
#include "sdk/nanoclient.h"
#include "sdk/nanobuffers.h"

#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <iostream>

int64_t now2()
{
	struct timeval tv;

#ifdef _WIN32
	struct __timeb64 timebuffer;
	_ftime64(&timebuffer);
	tv.tv_sec = timebuffer.time;
	tv.tv_usec = timebuffer.millitm * 1000;
#else
	gettimeofday(&tv, NULL);
#endif

	return (int64_t)(tv.tv_sec) * 1000 + (int64_t)(tv.tv_usec) / 1000;
}

// 继承 NanoClient, 实现其中的虚函数, 尤其 onMessage
class MyClient : public NanoClient
{
  public:
	// 接收到一个消息
	virtual void onMessage(uint8_t *data, int size, uint8_t fromIndex)
	{
		cout << "onMessage: size=" << size << ", fromIndex=" << (int)fromIndex << endl;

		int v1 = 0;

		NanoReader nanoReader(data, size);
		nanoReader.getInt(v1);

		if  (v1 == 1) {
			float v2 = 0;
			double v3 = 0;
			char str[1024];
			int str_len = 1024;
	
			nanoReader.getFloat(v2).getDouble(v3,3).getString(str, str_len);
	
			cout << "v1: " << v1 << ", v2: " << v2 << ", v3: " << v3 << ", " << str_len << ", " << string(str, str_len) << endl;
		}
		if (v1 == 255) {
			char str[1024];
			int str_len = 1024;
	
			nanoReader.getString(str, str_len);
	
			cout << "v1: " << v1 << ", " << string(str, str_len) << endl;
		}

	}

	// 响应其他客户端的 resync 请求
	virtual void onResync (uint8_t fromIndex)
	{
		NanoWriter nanoWriter;
		nanoWriter.putInt(255).putString("我的昵称等基本数据");
		send(nanoWriter.getBytes(), nanoWriter.getLength());

		cout << "onResync" << endl;
	}

	// 连接断开
	virtual void onDisconnected(int error)
	{
		if (error == 0) {
			if (disconnectedBySelf)
				cout << "主动断开" << endl;
			else
				cout << "对方断开" << endl;
		} else {
			// 错误代码具体参考 "Nanolink SDK 接口说明" 中 lastError 定义
			if (error == 501) {
				if (getInt ("last-time", -2) < 2000)
					cout << "超时断开, 可能是对方原因" << endl;
				else
					cout << "超时断开, 可能是己方原因" << endl;
			}
		}
	}

	// 当状态变化时
	virtual void onStatusChanged(string newStatus, string oldStatus)
	{
		cout << "状态由 " << oldStatus << " 变为 " << newStatus << endl;
	}

	// 连接建立
	virtual void onConnected()
	{
		cout << "onConnected: " << getInt ("client-index") << endl;
	}

};

MyClient client;

void nanoclient_test1 ()
{
	// 初始化代码
	// client.init("16438df79e6b09ac", 2);
	client.init("099235c844fd1855", 3); // 多人, 新版

	client.config("debug-level", "7");
	client.config("time-machine", "true"); // 设置为支持日志保存

	// 连接代码, 在 "开始" 按钮时触发
	client.connect(1);
	
	int64_t t1 = ::now2();
	int64_t t2 = ::now2();

	// 模拟一个 Update 循环
	while (true)
	{
		// 实际 cocos 游戏中, 在 Update 函数中调用 client.doUpdate
		client.doUpdate();

		// 每 500ms 发送一次数据
		if (client.isConnected() && ::now2() - t1 >= 500)
		{
			srand(::now2());

			NanoWriter nanoWriter;
			nanoWriter.putInt(1).putFloat(1.2345f).putDouble(2.34567, 3).putString("hello, 世界");

			// XXX 发送数据, 在操作或事件发生时
			cout << "发送数据: " << nanoWriter.getLength() << endl;
			client.send(nanoWriter.getBytes(), nanoWriter.getLength());

			t1 = ::now2();
		}

		// 连接中断 或 20s, 退出
		if (client.isDisconnect() || ::now2() - t2 > 20000)
		{
			cout << "连接中断或60s超时, 退出: " << client.getString("status") << endl;
			break;
		}

		// 等待 20ms, 相当于 50帧/s
		::usleep(20000);
	}

	// 断开, 在 "退出" 按钮触发
	if (!client.isDisconnect())
	{
		client.disconnect();
	}

	::usleep(100000); // 100ms

	// 保存日志, 在断开事件中 (onDisconnect)
	client.save("nanolink1.log");
}

int main(int argc, char **argv)
{
	nanoclient_test1 ();

	return 0;
}
