// 实例化 NanoClient
// 重写 onStatusChanged, onConnected, onDisconnected, onMessage 等接口
var myClient = new NanoClient ({
    // 接收到消息
    onMessage: function (data, fromIndex) {
        // nanobuffers 反序列化
        var nanoReader = new NanoReader ();
        var result = nanoReader.init (data).getInt().getInt().getFloat().getFloat (3).getString().getResult ();

        this.fireEvent ("nano:message", result);
    },

    // 连接断开
    onDisconnected: function (error) {
        console.log("连接已断开 onDisconnected: " + error);

        if (error == 0) {
            if (this.disconnectedBySelf)
                console.log ("主动断开");
            else
                console.log ("对方断开");
        } else {
			// 错误代码具体参考 "Nanolink SDK 接口说明" 中 lastError 定义
            if (error == 501) {
                if (this.getInt ("last-time", -2) < 2000)
                    console.log ("超时断开, 可能是对方原因");
                else
                    console.log ("超时断开, 可能是己方原因");
            }
        }

        this.fireEvent ("nano:disconnected");
    },

    // 状态变化
    onStatusChanged: function (newStatus, oldStatus) {
        console.log("状态变化：由 " + oldStatus + " 变为 " + newStatus);
        this.fireEvent ("nano:status-changed", newStatus);
    },

    onResync: function (fromIndex) {
        // 匹配成功时，同步玩家数据（如：昵称，位置，皮肤，...）给其他玩家
    },
    
    // 连接建立
    onConnected: function () {
        this.fireEvent ("nano:connected");
    },

    // 发送事件, 可以在其他地方用加上事件监听代码, 具体参考 cc.EventListener
    fireEvent: function (name, data) {
        var e = new cc.EventCustom(name);

        if (typeof data !== 'undefined')
            e.setUserData (data);

        cc.eventManager.dispatchEvent(e);
    }
});

var DemoLayer = cc.Layer.extend({
    labelRole:null,
    labelRecvMsg:null,
    labelStatus:null,
    labelLatency:null,
    ctor:function () {
        this._super();
        var size = cc.winSize;

        var order = 5;
        var x = 50;
        var y = size.height;
                                      
        y -= 50;
        this.addChild(this.newLabel (x, y, '多人对战'), order++);

        y -= 50;
        this.labelRole = this.newLabel (x, y);
        this.addChild(this.labelRole, order++);

        y -= 50;
        this.labelStatus = this.newLabel (x, y);
        this.addChild(this.labelStatus, order++);

        y -= 50;
        this.labelLatency = this.newLabel (x, y);
        this.addChild(this.labelLatency, order++);

        y -= 50;
        this.labelRecvMsg = this.newLabel (x, y);
        this.addChild(this.labelRecvMsg, order++);

        y -= 50;
        var btn = new ccui.Button();
        btn.setTouchEnabled(true);
        btn.setPressedActionEnabled(true);
        // btn.loadTextures("res/btn_a_1_fall.png", "res/btn_a_1_fall_on.png");
        // btn.setPosition(cc.p(size.width/2, 200));
        btn.x = x;
        btn.y = y;
        btn.setTitleText("开始匹配!");
        btn.setTitleFontSize(60);
        btn.setTitleFontName("Arial");
        btn.getTitleRenderer().setAnchorPoint(cc.p(0, 0));
        btn.getTitleRenderer().setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT);
        btn.addTouchEventListener (function (sender, type) {
            if(type == ccui.Widget.TOUCH_ENDED) {
                myClient.connect(1);
            }

        }.bind(this));
        this.addChild(btn);

        return true;
    },

    newLabel: function (x, y, txt) {
        if (typeof txt == 'undefined') txt = '';

        var label = new cc.LabelTTF(txt, "Arial", 30);
        label.setAnchorPoint(cc.p(0, 0));
        label.setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT);
        label.x = x;
        label.y = y;

        return label;
    },

    // 从 myClient 获得各个状态, 输出到屏幕左上角
    doUpdate: function () {
        this.labelStatus.setString("当前状态: " + myClient.getStatus ());

        // 连接成功后，输出状态信息
        if(!myClient.isConnected())
            return;
        
        var thePlayers = "";
        var latency = myClient.getInt("latency") / 1000;

        if (myClient.getInt("mode") == 2) {
            // 1 VS 1 延迟／2
            latency = latency / 2;
        } else if (myClient.getInt("mode") == 3) {
            thePlayers = ", 当前人数: " + myClient.getInt ("players", 5000);
        }

        var isMaster = myClient.getInt ("is-master");
        if (isMaster == 1)
          this.labelRole.setString("主场: " + myClient.getString ("client-id") + thePlayers);
        else
           this.labelRole.setString("客场: " + myClient.getString ("client-id") + thePlayers);

        this.labelLatency.setString("延迟: " + latency + "ms");

        // this.labelRecvMsg.setString("接收到新消息: " + recvMsg);
    }
});

var HelloWorldScene = cc.Scene.extend({
    demoLayer: null,
    last_send_time: 0,
    that: null,

    ctor:function() {
        this._super();

        this.demoLayer = new DemoLayer();
    },

    onEnter:function () {
        this._super();

        this.addChild(this.demoLayer);

        // 初始化, 多人模式 Demo
        myClient.init("ed939044b69bfdd7", 3);
       
        // 配置日志输出模式                          
        myClient.config ("debug-level", "7");

        // 开启 保存存档功能
        // myClient.config ("time-machine", "true");

        // 设置每帧回调
        this.schedule(this.onUpdate, 0);

        // 创建监听 myClient onXXX 事件监听器
        that = this;
        this.onEventListener();
    },

    onUpdate:function (dt) {
        var t = Date.now();

        // 数据和状态处理循环
        myClient.doUpdate ();

        this.demoLayer.doUpdate ();
        
        // 每 1 秒发送一次数据
        if (myClient.isConnected() && t - this.last_send_time >= 1000) {
            // nanobuffers 序列化
            var nanoWriter = new NanoWriter();
            nanoWriter.putInt(1).putInt(-10000).putFloat(-1.234567).putFloat (2.3456789, 3).putString("hello, 世界");

            var byteArray = nanoWriter.getBytes ();
            myClient.send(byteArray);

            console.log ("发送的数据：1 -10000 -1.234567 2.3456789 hello, 世界");

            this.last_send_time = t;
        }
    },

    // myClient onXXX 事件监听器
    onEventListener: function() {
        // nano:status-changed
        // nano:connected
        // nano:disconnected
        // nano:message

        var message = cc.EventListener.create({
            event: cc.EventListener.CUSTOM,
            eventName: "nano:message",
            callback: function(event) {
                var userData = event.getUserData();

                var str = "";
                for (var i in userData) {
                    str += (userData[i] + " ");
                }

                console.log("接收到数据：" + str);
                // that.demoLayer.recvMsg = str;
            }
        });

        // 添加侦听器事件
        cc.eventManager.addListener(message, 1);
    }
});