function NanoWriter (params) {
    var newWriter = {
        data: new Uint8Array (64),
        pos: 0,

        _putUint8: function (v) {
            this.resize (this.pos+1);

            this.data [this.pos++] = v;
        },

        _putUint64: function (v) {
            if (v <= 240) {
                this._putUint8 (v);
            } else if (v <= 2287) {
                this._putUint8 (((v - 240) / 256 + 241));
                this._putUint8 (((v - 240) % 256));
            } else if (v <= 67823) {
                this._putUint8 (249);
                this._putUint8 (((v - 2288) / 256));
                this._putUint8 (((v - 2288) % 256));
            } else if (v <= 16777215) {
                this._putUint8 (250);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
            } else if (v <= 4294967295) {
                this._putUint8 (251);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
                this._putUint8 (((v >> 24) & 0xFF));
            } else if (v <= 1099511627775) {
                this._putUint8 (252);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
                this._putUint8 (((v >> 24) & 0xFF));
                this._putUint8 (((v >> 32) & 0xFF));
            } else if (v <= 281474976710655) {
                this._putUint8 (253);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
                this._putUint8 (((v >> 24) & 0xFF));
                this._putUint8 (((v >> 32) & 0xFF));
                this._putUint8 (((v >> 40) & 0xFF));
            } else if (v <= 72057594037927935) {
                this._putUint8 (254);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
                this._putUint8 (((v >> 24) & 0xFF));
                this._putUint8 (((v >> 32) & 0xFF));
                this._putUint8 (((v >> 40) & 0xFF));
                this._putUint8 (((v >> 48) & 0xFF));
            } else {
                this._putUint8 (255);
                this._putUint8 ((v & 0xFF));
                this._putUint8 (((v >> 8) & 0xFF));
                this._putUint8 (((v >> 16) & 0xFF));
                this._putUint8 (((v >> 24) & 0xFF));
                this._putUint8 (((v >> 32) & 0xFF));
                this._putUint8 (((v >> 40) & 0xFF));
                this._putUint8 (((v >> 48) & 0xFF));
                this._putUint8 (((v >> 56) & 0xFF));
            }
        },

        _putInt64: function (v) {
            var v1 = Math.abs (v) << 1;
            var v2 = (v >= 0 ? 0 : 1);
            this._putUint64 (v1 - v2);
        },

        _putFloat: function (v) {
            var buffer = new ArrayBuffer(4);
            new Float32Array(buffer) [0] = v;

            this._putBytes (new Uint8Array(buffer));
        },

        _putDouble: function (v) {
            var buffer = new ArrayBuffer(8);
            new Float64Array(buffer) [0] = v;

            this._putBytes (new Uint8Array(buffer));
        },

        _putBytes: function (v) {
            this.resize (this.pos+v.length);

            for (var i = 0; i < v.length; i ++)
                this.data [this.pos+i] = v[i];

            this.pos += v.length;
        },

        _putStringToUTF8Bytes: function (str) {
            for (var i=0; i < str.length; i++) {
                var charcode = str.charCodeAt(i);
                if (charcode < 0x80)
                    this._putUint8 (charcode);
                else if (charcode < 0x800) {
                    this._putUint8 (0xc0 | (charcode >> 6)); 
                    this._putUint8 (0x80 | (charcode & 0x3f));
                }
                else if (charcode < 0xd800 || charcode >= 0xe000) {
                    this._putUint8 (0xe0 | (charcode >> 12));
                    this._putUint8 (0x80 | ((charcode>>6) & 0x3f)); 
                    this._putUint8 (0x80 | (charcode & 0x3f));
                }
                // surrogate pair
                else {
                    i++;
                    // UTF-16 encodes 0x10000-0x10FFFF by
                    // subtracting 0x10000 and splitting the
                    // 20 bits of 0x0-0xFFFFF into two halves
                    charcode = 0x10000 + (((charcode & 0x3ff)<<10)
                            | (str.charCodeAt(i) & 0x3ff));
                    this._putUint8(0xf0 | (charcode >>18));
                    this._putUint8 (0x80 | ((charcode>>12) & 0x3f));
                    this._putUint8 (0x80 | ((charcode>>6) & 0x3f));
                    this._putUint8 (0x80 | (charcode & 0x3f));
                }
            }
        },

        // 返回当前空间大小
        resize: function (n) {
            if (n > this.data.length) {
                var newSize = this.data.length*2;
                if (newSize < n)
                    newSize = n;

                var newData = new Uint8Array (newSize);
                for (var i = 0; i < this.pos; i ++)
                    newData [i] = this.data [i];

                this.data = newData;
            }

            return this.data.length;
        },

        fit: function () {
            if (this.pos == 0) {
                this.data = NULL;
                return;
            }

            var newData = new Uint8Array (this.pos);
            for (var i = 0; i < this.pos; i ++)
                newData [i] = this.data [i];

            this.data = newData;
        },

        getBytesLeft: function () {
            return this.data.length - this.pos;
        },

        getLength: function () {
            return this.pos;
        },

        getBytes: function () {
            this.fit ();
            return this.data;
        },

        putInt: function (v) {
            this._putInt64 (v);
            return this;
        },

        putBytes: function (v) {
            this._putUint64 (v.length);
            this._putBytes (v, v.length);
            return this;
        },

        putString: function (v) { // UTF8
            this._putUint64 (v.length);
            this._putStringToUTF8Bytes (v);
            return this;
        },

        putFloat: function (v, precision) {
            precision = (typeof precision !== 'undefined') ?  precision : -1;
            if (precision < 0)
                this._putFloat (v);
            else
                this._putInt64 ((v*Math.pow(10, precision+1) + 5)/10);
            return this;
        },

        putDouble: function (v, precision) {
            precision = (typeof precision !== 'undefined') ?  precision : -1;
            if (precision == -1)
                this._putDouble (v);
            else
                this._putInt64 ((v*Math.pow(10, precision+1) + 5)/10);
            return this;
        }
    }

    if (typeof params == 'object') {
        for (var attrname in params) {
            newWriter[attrname] = params[attrname];
        }
    }

    return newWriter;
}

function NanoReader (params) {
    var newReader = {
        data: null,
        pos: 0,

        result: {},
        result_index: 0,

        _getUint8: function () {
            if (this.getBytesLeft () < 1) {
                // 抛出异常
                return 0;
            }

            return this.data [this.pos++];
        },

        _getUint64: function () {
            var a0 = this._getUint8 ();
            if (a0 < 241) {
                return a0;
            }

            var a1 = this._getUint8 ();
            if (a0 >= 241 && a0 <= 248) {
                return (240 + 256 * (a0 - 241) + a1);
            }

            var a2 = this._getUint8 ();
            if (a0 == 249) {
                return (2288 + 256 * a1 + a2);
            }

            var a3 = this._getUint8 ();
            if (a0 == 250) {
                return a1 + ((a2) << 8) + ((a3) << 16);
            }

            var a4 = this._getUint8 ();
            if (a0 == 251) {
                return a1 + ((a2) << 8) + ((a3) << 16) + ((a4) << 24);
            }

            var a5 = this._getUint8 ();
            if (a0 == 252) {
                return a1 + ((a2) << 8) + ((a3) << 16) + ((a4) << 24) + ((a5) << 32);
            }

            var a6 = this._getUint8 ();
            if (a0 == 253) {
                return a1 + ((a2) << 8) + ((a3) << 16) + ((a4) << 24) + ((a5) << 32)
                        + ((a6) << 40);
            }

            var a7 = this._getUint8 ();
            if (a0 == 254) {
                return a1 + ((a2) << 8) + ((a3) << 16) + ((a4) << 24) + ((a5) << 32)
                        + ((a6) << 40) + ((a7) << 48);
            }

            var a8 = this._getUint8 ();
            if (a0 == 255) {
                return a1 + ((a2) << 8) + ((a3) << 16) + ((a4) << 24) + ((a5) << 32)
                        + ((a6) << 40) + ((a7) << 48)  + ((a8) << 56);
            }

            // throw new IndexOutOfRangeException("getUInt64 () failure: " + a0);
            return 0;
        },

        _getInt64: function () {
            var v = this._getUint64 ();
            var v1 = v & 1;
            var v2 = (v >> 1);
            if (v1 == 0)
                return v2;
            else
                return -(v2+v1);
        },

        _getFloat: function () {
            var buffer = this._getBytes (4);
            return new Float32Array (buffer.buffer) [0];
        },

        _getDouble: function () {
            var buffer = this._getBytes (8);
            return new Float64Array (buffer.buffer) [0];
        },

        //
        _getBytes: function (len) {
            if (len > this.getBytesLeft ()) {
                // 抛出异常
                return null;
            }

            var buf = new Uint8Array (len);
            for (var i = 0; i < len; i ++)
                buf [i] = this.data [this.pos +i];

            this.pos += len;

            return buf;
        },

        _getStringFromUTF8Bytes: function (len) {
            var str = '';

            while (str.length < len) {
                var value = this.data[this.pos ++];

                if (value < 0x80) {
                    str += String.fromCharCode(value);
                } else if (value > 0xBF && value < 0xE0) {
                    str += String.fromCharCode(((value & 0x1F) << 6)
                            | (this.data[this.pos ++] & 0x3F));
                } else if (value > 0xDF && value < 0xF0) {
                    str += String.fromCharCode(((value & 0x0F) << 12)
                            | ((this.data[this.pos ++] & 0x3F) << 6)
                            | (this.data[this.pos ++] & 0x3F));
                } else {
                    // surrogate pair
                    var charCode = (((value & 0x07) << 18)
                            | ((this.data[this.pos ++] & 0x3F) << 12)
                            | ((this.data[this.pos ++] & 0x3F) << 6)
                            | ((this.data[this.pos ++] & 0x3F))) - 0x010000;

                    str += String.fromCharCode(charCode >> 10 | 0xD800, charCode & 0x03FF | 0xDC00); 
                }
            }

            return str;
        },

        init: function (v) {
            this.data = v;
            this.pos = 0;

            this.result_index = 0;
            this.result = {};

            return this;
        },

        getResult: function () {
            return this.result;
        },

        getBytesLeft: function () {
            return this.data.length - this.pos;
        },

        getLength: function () {
            return this.pos;
        },

        getInt: function () {
            this.result[this.result_index++] = this._getInt64 ();

            return this;
        },

        getFloat: function (precision) {
            precision = (typeof precision !== 'undefined') ?  precision : -1;
            if (precision < 0)
                this.result[this.result_index++] = this._getFloat ();
            else
                this.result[this.result_index++] = (this._getInt64 () / Math.pow(10, precision));

            return this;
        },

        getDouble: function (precision) {
            precision = (typeof precision !== 'undefined') ?  precision : -1;
            if (precision < 0)
                this.result[this.result_index++] = this._getDouble ();
            else
                this.result[this.result_index++] = (this._getInt64 () / Math.pow(10, precision));

            return this;
        },

        getBytes: function () {
            var len = this._getUint64 ();

            this.result [this.result_index++] = this._getBytes (len);

            return this;
        },

        getString: function () {
            var len = this._getUint64 ();

            this.result [this.result_index++] = this._getStringFromUTF8Bytes (len);

            return this;
        }
    }

    if (typeof params == 'object') {
        for (var attrname in params) {
            newReader [attrname] = params[attrname];
        }
    }

    return newReader;
}

function nanobuffer_test1 () {
    var nanoWriter = new NanoWriter ({
        // 扩展 put 接口
        putVec3: function(v,precision) {
            this.putFloat (v.x, precision).putFloat (v.y, precision).putFloat (v.z, precision);

            return this;
        }
    });
    var nanoReader = new NanoReader ({
        // 扩展 get 接口
        getVec3: function(precision) {
            precision = (typeof precision !== 'undefined') ?  precision : -1;
            var v = {};
            if (precision < 0) {
                v.x = this._getFloat ();
                v.y = this._getFloat ();
                v.z = this._getFloat ();
            } else {
                v.x = (this._getInt64 () / Math.pow(10, precision));
                v.y = (this._getInt64 () / Math.pow(10, precision));
                v.z = (this._getInt64 () / Math.pow(10, precision));
            }
            this.result[this.result_index++] = v;

            return this;
        }
    });

    nanoWriter.putInt (123).putInt(-1000).putFloat (1.2345).putDouble (3.1415927, 3).putVec3({x:1.0,y:2.3456789,z:3.1415926}, 3).putString ("hello, 中国");

    var result= nanoReader.init (nanoWriter.getBytes ()).getInt().getInt().getFloat().getDouble(3).getVec3(3).getString ().getResult ();

    // result[0] == 123
    // result[1] == -1000
    // result[2] == 1.2345
    // result[3] == 3.142
    // result[4] == "hello, 中国"
    // result[5] == {x:1.0, y:2.346, z:3.142}
    return result;
}

nanobuffer_test1 ();
