#include "nanolink_js.h"
#include "nanolink.h"
#include "cocos2d.h"
#include "ScriptingCore.h"
#include "jsb_helper.h"
#include "jsapi.h"
#include "jsfriendapi.h"
#include "scripting/js-bindings/manual/js_bindings_config.h"
#include "scripting/js-bindings/manual/js_manual_conversions.h"
#include <vector>

void nanolink_js::registerJSFunctions(JSContext * cx, JS::HandleObject global) {
    JS_DefineFunction(cx, global, "nano_init", nano_init_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_config", nano_config_, 2, JSPROP_READONLY | JSPROP_PERMANENT);

    JS_DefineFunction(cx, global, "nano_connect", nano_connect_, 3, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_connect2", nano_connect2_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_disconnect", nano_disconnect_, 0, JSPROP_READONLY | JSPROP_PERMANENT);

    JS_DefineFunction(cx, global, "nano_send", nano_send_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_sendmark", nano_sendmark_, 1, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_recv", nano_recv_, 3, JSPROP_READONLY | JSPROP_PERMANENT);

    JS_DefineFunction(cx, global, "nano_update", nano_update_, 4, JSPROP_READONLY | JSPROP_PERMANENT);

    JS_DefineFunction(cx, global, "nano_getInt", nano_getInt_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
    JS_DefineFunction(cx, global, "nano_getString", nano_getString_, 3, JSPROP_READONLY | JSPROP_PERMANENT);

    // JS_DefineFunction(cx, global, "nano_load", nano_load_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
    // JS_DefineFunction(cx, global, "nano_save", nano_save_, 2, JSPROP_READONLY | JSPROP_PERMANENT);
}

bool nano_init_ (JSContext * cx, uint32_t argc, jsval *vp) {
    const char *appKey;
    int32_t mode;

    JSB_PRECONDITION2( argc == 2, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_charptr( cx, args.get(0), &appKey );
    ok &= jsval_to_int(cx, args.get(1), &mode);
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int32_t ret = nano_init(appKey, mode);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_config_ (JSContext * cx, uint32_t argc, jsval *vp) {
    const char *name;
    const char *value;

    JSB_PRECONDITION2( argc == 2, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_charptr( cx, args.get(0), &name );
    ok &= jsval_to_charptr( cx, args.get(1), &value );
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int ret = nano_config(name, value);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_connect_ (JSContext * cx, uint32_t argc, jsval *vp) {
    int32_t level;
    int32_t mode;
    int32_t timeout = 15000;

    JSB_PRECONDITION2( argc >= 1, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_int( cx, args.get(0), &level );
    if (argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &mode );
    }
    if (argc > 2) {
        ok &= jsval_to_int( cx, args.get(2), &timeout );
    }
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int ret = nano_connect((uint8_t)level, (uint8_t)mode, timeout);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_connect2_ (JSContext * cx, uint32_t argc, jsval *vp) {
    const char* group;
    int32_t timeout = 15000;

    JSB_PRECONDITION2( argc >= 1, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_charptr( cx, args.get(0), &group );
    if (argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &timeout );
    }
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int ret = nano_connect2(group, timeout);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_disconnect_ (JSContext * cx, uint32_t argc, jsval *vp) {
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
    int ret = nano_disconnect();
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_send_ (JSContext * cx, uint32_t argc, jsval *vp) {
    std::vector<int> data;
    int32_t strength=255;

    // JSB_PRECONDITION2( argc >= 1, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_std_vector_int( cx, args.get(0), &data );
    if (argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &strength );
    }

    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");
    
    int ret = 0;
    int len = data.size();
    if (len > 0) {
        uint8_t *buf = new uint8_t[len];
        for (int i=0; i<len; i++) {
            buf[i] = (uint8_t)data.at(i);
        }
        ret = nano_send(buf, len, (uint8_t)strength);
        delete [] buf;
    }
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_sendmark_ (JSContext * cx, uint32_t argc, jsval *vp) {
    int32_t mark=0;

    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_int( cx, args.get(0), &mark );

    int ret = nano_sendmark((uint8_t)mark);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_recv_ (JSContext * cx, uint32_t argc, jsval *vp) {
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;

    uint8_t playerIndex;
    int32_t len = nano_recv(NULL, 0, playerIndex);

    if (len < 0) {
        args.rval().setUndefined();
        return true;
    }

    // len >= 0, ...

    uint8_t *data = data = new uint8_t[len+1];
    memset(data, 0, len+1);
    
    len = nano_recv(data, len, playerIndex);
    
    std::vector<int> buf;
    for (int i=0; i<len; i++) {
        buf.push_back(data[i]);
    }

    JS::RootedObject array(cx, JS_NewArrayObject(cx, 2));
    JS::RootedValue js_msg(cx, std_vector_int_to_jsval(cx, buf));
    JS::RootedValue js_playerIndex(cx, INT_TO_JSVAL(playerIndex));
    ok &= JS_SetElement(cx, array, 0, js_msg);
    ok &= JS_SetElement(cx, array, 1, js_playerIndex);
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");
    args.rval().set(OBJECT_TO_JSVAL(array));

    delete [] data;

    return true;
}

bool nano_update_ (JSContext * cx, uint32_t argc, jsval *vp) {
    int32_t x;
    int32_t y;
    int32_t z = 0;
    int32_t k = 1;

    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_int( cx, args.get(0), &x );
    ok &= jsval_to_int( cx, args.get(1), &y );
    if (argc > 1) {
        ok &= jsval_to_int( cx, args.get(2), &z );
    }
    if (argc > 2) {
        ok &= jsval_to_int( cx, args.get(3), &k );
    }
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int ret = nano_update((uint8_t)x, (uint8_t)y, (uint8_t)z, (uint8_t)k);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_getInt_ (JSContext * cx, uint32_t argc, jsval *vp) {
    const char *name;
    int32_t n = -1;

    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    ok &= jsval_to_charptr( cx, args.get(0), &name );
    if(argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &n );
    }
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");

    int ret = nano_getInt(name, n);
    args.rval().set(INT_TO_JSVAL(ret));
    return true;
}

bool nano_getString_ (JSContext * cx, uint32_t argc, jsval *vp) {
    const char *name;
    
    JSB_PRECONDITION2( argc == 1, cx, false, "Invalid number of arguments" );
    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
    
    bool ok = true;
    ok &= jsval_to_charptr( cx, args.get(0), &name );
    JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");
    
    char *data = NULL;
    int32_t len = nano_getString(name, NULL, 0);
    if (len > 0) {
        data = new char[len+1];
        memset(data, 0, len+1);
        
        len = nano_getString(name, data, len+1);
    }
   
    args.rval().set(std_string_to_jsval(cx, len > 0 ? data : ""));
    
    if (data)
        delete [] data;
    
    return true;
}

/*
bool nano_load_ (JSContext * cx, uint32_t argc, jsval *vp) {
    int size = -1;

    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    if(argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &size );
    }

    if(size == -1) {
        const char *dataPath;
        ok &= jsval_to_int( cx, args.get(0), &dataPath );

        int ret = nano_load(dataPath, size);
        args.rval().set(INT_TO_JSVAL(ret));
    } else {
        if (size >= 0) {
            uint8_t *data = new uint8_t[size];
            memset(data, 0, size);

            nano_load(data, size);

            std::vector<int> buf;
            for (int i=0; i<size; i++) {
                buf.push_back(data[i]);
            }

            JS::RootedObject array(cx, JS_NewArrayObject(cx, 1));
            JS::RootedValue js_msg(cx, std_vector_int_to_jsval(cx, buf));
            ok &= JS_SetElement(cx, array, 0, js_msg);
            JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");
            args.rval().set(OBJECT_TO_JSVAL(array));
        }
        else args.rval().setUndefined();

        delete [] data;
    }

    return true;
}

bool nano_save_ (JSContext * cx, uint32_t argc, jsval *vp) {
    int size = -1;

    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);

    bool ok = true;
    if(argc > 1) {
        ok &= jsval_to_int( cx, args.get(1), &size );
    }

    if(size == -1) {
        const char *dataPath;
        ok &= jsval_to_int( cx, args.get(0), &dataPath );

        int ret = nano_save(dataPath, size);
        args.rval().set(INT_TO_JSVAL(ret));
    } else {
        if (size >= 0) {
            uint8_t *data = new uint8_t[size];
            memset(data, 0, size);

            nano_save(data, size);
            
            std::vector<int> buf;
            for (int i=0; i<size; i++) {
                buf.push_back(data[i]);
            }

            JS::RootedObject array(cx, JS_NewArrayObject(cx, 1));
            JS::RootedValue js_msg(cx, std_vector_int_to_jsval(cx, buf));
            ok &= JS_SetElement(cx, array, 0, js_msg);
            JSB_PRECONDITION2(ok, cx, false, "Error processing arguments");
            args.rval().set(OBJECT_TO_JSVAL(array));
        }
        else args.rval().setUndefined();

        delete [] data;
    }

    return true;
}
*/
