require("nanoclient")
require("nanobuffers")

local nanolink_demo = {}

local myClient = NanoClient.extend({
    -- 接收到消息
    onMessage = function(data, fromIndex)
        print("onMessage, fromIndex:", fromIndex)

        -- print(string.byte (data, 1, string.len(data)))

        -- nanobuffers 反序列化
        local nanoReader = NanoReader:create(data)
        local result = nanoReader:getInt():getInt():getFloat(4):getFloat (3):getString():getResult ()
        
        -- 接收到数据
        for i,v in ipairs(result) do
            print(i,v)
        end
    end,

    -- 连接断开
    onDisconnected = function (error)
        print("连接已断开 onDisconnected, error: ", error)

        if (error == 0) then
            if (NanoClient.disconnectedBySelf) then
                print ("主动断开")
            else
                print ("对方断开")
            end
        else
			-- 错误代码具体参考 "Nanolink SDK 接口说明" 中 lastError 定义
            if (error == 501) then
                if (NanoClient.getInt ("last-time", -2) < 2000) then
                    print ("超时断开, 可能是对方原因")
                else
                    print ("超时断开, 可能是己方原因")
                end
            end
        end
    end,

    -- 状态变化
    onStatusChanged = function(newStatus, oldStatus)
        print(string.format("onStatusChanged newStatus=%s ; oldStatus=%s", newStatus, oldStatus))
    end,

    -- 匹配成功
    onConnected = function ()
        -- body
    end
})

-- NanoLink 参数
local lastSendTime = os.time()
local lastStatus = 0

-- 游戏模式
-- mode = 0: 保留
-- mode = 1: 单人, 用于记录与回放
-- mode = 2: 1对1联网方式(缺省)
-- mode = 3: 多人联网方式
local game_mode = 2
local nano_appkey = "8a674b910fc6ab7a"

-- scheduler:scheduleScriptFunc 参数 单位秒 0:每一帧
local interval = 1

local function update()
    local t = os.time()
    myClient.doUpdate()

    if(myClient.isConnected() and (t - lastSendTime) >= 1) then
        lastSendTime = t

        -- local data = ""
        -- for i=0,255 do
        --     data = data..string.char(i)
        -- end

        -- nanobuffers 序列化
        local nanoWriter = NanoWriter:create()
        local bytes = nanoWriter:putInt(1):putInt(-10000):putFloat(-1.234567, 4):putFloat (2.3456789, 3):putString("hello, 世界"):getBytes()
        myClient.send(bytes)
    end
end

function nanolink_demo.start()
    -- nanolink 初始化
    myClient.init(nano_appkey, game_mode)

    -- nanolink 打开调试日志
    myClient.config("debug-level", "7")

    -- nanolink 开始匹配
    myClient.connect(1)

    -- 定时器
    local scheduler = cc.Director:getInstance():getScheduler()
    scheduler:scheduleScriptFunc(update, interval, false)
end

return nanolink_demo
