require "bit"
----------------NanoWriter------------------
_G.NanoWriter = {}

function NanoWriter:create()
    local obj = {}
    setmetatable(obj, {__index = NanoWriter})
    return obj:init()
end

function NanoWriter:init()
    self.data = {}
    self.pos = 1
    return self
end

function NanoWriter:_putUint8(v)
    self.data[self.pos] = v
    self.pos = self.pos + 1
end

function NanoWriter:_putUint64(v)
    if v <= 240 then
        self:_putUint8(v)
    elseif v <= 2287 then
        self:_putUint8((v - 240) / 256 + 241)
        self:_putUint8((v - 240) % 256)
    elseif v <= 67823 then
        self:_putUint8(249)
        self:_putUint8((v - 2288) / 256)
        self:_putUint8((v - 2288) % 256)
    elseif v <= 16777215 then
        self:_putUint8(250)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
    elseif v <= 4294967295 then
        self:_putUint8(251)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 24), 0xFF))
    elseif v <= 1099511627775 then
        self:_putUint8(252)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 24), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 32), 0xFF))
    elseif v <= 281474976710655 then
        self:_putUint8(253)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 24), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 32), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 40), 0xFF))
    elseif v <= 72057594037927935 then
        self:_putUint8(254)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 24), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 32), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 40), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 48), 0xFF))
    else
        self:_putUint8(255)
        self:_putUint8(bit.band(v, 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 8), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 16), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 24), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 32), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 40), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 48), 0xFF))
        self:_putUint8(bit.band(bit.rshift(v, 56), 0xFF))
    end
end

function NanoWriter:_putInt64(v)
    local v1 = math.abs(v) * 2
    local v2 
    if v >= 0 then
        v2 = 0
    else
        v2 = 1
    end
    self:_putUint64(v1 - v2)
end

function NanoWriter:putInt(v)
    self:_putInt64(v)
    return self
end

function NanoWriter:putFloat(v, precision)
    if type(precision) ~= "number" or precision < 0 then
        return -1
    end

    self:_putUint64(precision)

    local value = v * math.pow(10, precision)
    self:_putInt64(value)
    return self
end

function NanoWriter:putDouble(v, precision)
    if type(precision) ~= "number" or precision < 0 then
        return -1
    end

    self:_putUint64(precision)

    self:_putInt64((v * math.pow(10, precision) + 5) / 10)
    return self
end

function NanoWriter:putString(v)
    self:_putUint64(string.len(v))
    for i=1,string.len(v) do
        local str = string.byte(v, i, i)
        self:_putUint8(str)
    end
    return self
end

function NanoWriter:getBytes(v)
    local _data = ""
    for i,v in ipairs(self.data) do
        _data = _data..string.char(v)
    end
    return _data
end

----------------NanoReader------------------
_G.NanoReader = {}

function NanoReader:create(v)
    local obj = {}
    setmetatable(obj, {__index = NanoReader})
    return obj:init(v)
end

function NanoReader:init(v)
    self.data = {}
    for i=1,string.len(v) do
        local s = string.byte(v, i, i)
        self.data[i] = s
    end

    self.pos = 1
    self.result_index = 1
    self.result = {}
    return self
end

function NanoReader:getBytesLeft()
    return #self.data - self.pos
end

function NanoReader:_getUint8()
    local v = self.data[self.pos]
    self.pos = self.pos + 1
    return v
end

function NanoReader:_getUint64()
    local a0 = self:_getUint8()
    if a0 < 241 then
        return a0
    end

    local a1 = self:_getUint8()
    if a0 >= 241 and a0 <= 248 then
        return 240 + 256 * (a0 - 241) + a1
    end

    local a2 = self:_getUint8()
    if a0 == 249 then
        return 2288 + 256 * a1 + a2
    end

    local a3 = self:_getUint8()
    if a0 == 250 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16)
    end

    local a4 = self:_getUint8()
    if a0 == 251 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16) + bit.lshift(a4, 24)
    end

    local a5 = self:_getUint8()
    if a0 == 252 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16) + bit.lshift(a4, 24) + bit.lshift(a5, 32)
    end

    local a6 = self:_getUint8()
    if a0 == 253 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16) + bit.lshift(a4, 24) + bit.lshift(a5, 32) + bit.lshift(a6, 40)
    end

    local a7 = self:_getUint8()
    if a0 == 254 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16) + bit.lshift(a4, 24) + bit.lshift(a5, 32) + bit.lshift(a6, 40) + bit.lshift(a7, 48)
    end

    local a8 = self:_getUint8()
    if a0 == 255 then
        return a1 + bit.lshift(a2, 8) + bit.lshift(a3, 16) + bit.lshift(a4, 24) + bit.lshift(a5, 32) + bit.lshift(a6, 40) + bit.lshift(a7, 48) + bit.lshift(a8, 56)
    end

    return 0
end

function NanoReader:_getInt64()
    local v = self:_getUint64()
    local v1 = bit.band(v, 1)
    local v2 = bit.rshift(v, 1)
    if v1 == 0 then
        return v2
    else
        return -(v2 + v1)
    end
end

function NanoReader:getInt()
    local value = self:_getInt64()
    self:setResult(value)
    return self
end

function NanoReader:getFloat()
   local precision = self:_getUint64()

    local value = self:_getInt64() / math.pow(10, precision)
    self:setResult(value)
    return self
end

function NanoReader:getDouble()
    local precision = self:_getUint64()

    local value = self:_getInt64() / math.pow(10, precision)
    self:setResult(value)
    return self
end

function NanoReader:getString()
    local len = self:_getUint64()
    local str = ""

    while string.len(str) < len do
        local s = self:_getUint8()
        str = str..string.char(s)
    end
    
    self:setResult(str)
    return self
end

function NanoReader:setResult(v)
    self.result[self.result_index] = v
    self.result_index = self.result_index + 1
end

function NanoReader:getResult()
    return self.result
end

--[[
    教程:
        local writer = NanoWriter:create()
        local bytes = writer:putInt(121):putFloat(3.14152, 5):putString("我是测试代码"):getBytes()
        
        local reader = NanoReader:create(bytes)
        local result = reader:getInt():getFloat():getString():getResult() 

        for k,v in pairs(result) do
            print(k,v)
        end
]]
