#include "nanolink_lua.h"
#include "nanolink.h"
#include <stdlib.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif
    
#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"

static int nanolink_init(lua_State *L) {
    size_t l;
    int32_t mode = 2;
    if (lua_gettop(L) > 1) {
        mode = luaL_checkinteger(L, 2);
    }
    int32_t ret = nano_init(luaL_checklstring(L, 1, &l), mode);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_config(lua_State *L) {
    size_t l;
    int32_t ret = nano_config(luaL_checklstring(L, 1, &l), luaL_checklstring(L, 2, &l));
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_connect(lua_State *L) {
    int32_t timeout=15000;
    uint8_t mode = 0;
    if (lua_gettop(L) > 1) {
        timeout = luaL_checkinteger(L,2);
    }
    if (lua_gettop(L) > 2) {
        mode = (uint8_t)luaL_checkinteger(L,2);
        timeout = luaL_checkinteger(L,3);
    }
    int32_t ret = nano_connect((uint8_t)luaL_checkinteger(L, 1), mode, timeout);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_connect2(lua_State *L) {
    const char* group = NULL;
    int32_t timeout=15000;
    if (lua_gettop(L) > 0) {
        size_t l;
        group = luaL_checklstring(L,1,&l);
    }
    if (lua_gettop(L) > 1) {
        timeout = luaL_checkinteger(L,2);
    }
    int32_t ret = nano_connect2(group, timeout);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_disconnect(lua_State *L) {
    int32_t ret = nano_disconnect();
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_send(lua_State *L) {
    size_t len;
    uint8_t strengh = 255;
    const char *data = lua_tolstring(L, 1, &len);

    if (lua_gettop(L) > 1) {
        strengh = luaL_checkinteger(L, 2);
    }
     
    int ret = nano_send((uint8_t*)data, len, strengh);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_sendmark(lua_State *L) {
    uint8_t mark = 0;
    if (lua_gettop(L) > 0) {
        mark = (uint8_t)luaL_checkinteger(L, 1);
    }
    int32_t ret = nano_sendmark(mark);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_recv(lua_State *L) {
    uint8_t playerIndex;
    int32_t len = nano_recv(NULL, 0, playerIndex);
    if (len < 0)
        lua_pushnil(L);

    if (len == 0)    
        lua_pushlstring(L, NULL, 0);

    if (len > 0) {
        uint8_t *data = new uint8_t[len];
        memset(data, 0, sizeof(data));

        len = nano_recv(data, len, playerIndex);

        lua_pushlstring(L, (char*)data, len);

        lua_pushinteger(L, playerIndex);

        delete []data;

        return 2;
    }

    return 1;
}

static bool nonolink_update(lua_State *L) {
    uint8_t z = 0;
    uint8_t k = 1;

    if (lua_gettop(L) > 2) {
        z = (uint8_t)luaL_checkinteger(L,3);
    }

    if (lua_gettop(L) > 3) {
        z = (uint8_t)luaL_checkinteger(L,3);
        k = (uint8_t)luaL_checkinteger(L,4);
    }
    
    int32_t ret = nano_update((uint8_t)luaL_checkinteger(L,1), (uint8_t)luaL_checkinteger(L,2), z, k);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_getInt(lua_State *L) {
    size_t l;
    int32_t n = -1;
    if (lua_gettop(L) > 1) {
        n = luaL_checkinteger(L, 2);
    }
    int32_t ret = nano_getInt(luaL_checklstring(L, 1, &l), n);
    lua_pushinteger(L,ret);
    return 1;
}

static int nanolink_getString(lua_State *L) {
    const char *name;
    size_t l;
    if (lua_gettop(L) > 0) {
        name = luaL_checklstring(L,1,&l);
    }
    
    char *data = NULL;
    int32_t len = nano_getString(name, NULL, 0);
    if (len > 0) {
        data = new char[len+1];
        memset(data, 0, len+1);
    }
    len = nano_getString(name, data, len+1);
    lua_pushlstring(L, data, len);

    if (data) {
        delete []data;
    }
    return 1;
}

static luaL_Reg nanolink_func[] = {
    {"init",nanolink_init},
    {"config",nanolink_config},
    {"connect",nanolink_connect},
    {"connect2",nanolink_connect2},
    {"disconnect",nanolink_disconnect},
    {"send",nanolink_send},
    {"sendmark",nanolink_sendmark},
    {"recv",nanolink_recv},
    {"getInt",nanolink_getInt},
    {"getString",nanolink_getString},
    {NULL,NULL}
};

#if LUA_VERSION_NUM > 501
//5.2.x
int nanolink(lua_State *L)
{
    luaL_newlib(L,nanolink_func);
    return 1;
}
void nanolink_lua_init(lua_State* L)
{
    luaL_requiref(L, "nanolink", nanolink, 1);
}
#else
//5.1.x
void nanolink_lua_init(lua_State* L)
{
    luaL_register(L,"nanolink",nanolink_func);
}
#endif
    
#ifdef __cplusplus
}
#endif
